@@ -1,16 +1,11 @@
 "use strict";
 
 function run_test() {
-  var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
-
-  var newURI = ios.newURI("http://foo.com");
+  var newURI = Services.io.newURI("http://foo.com");
 
   var success = false;
   try {
-    newURI = newURI
-      .mutate()
-      .setSpec("http: //foo.com")
-      .finalize();
+    newURI = newURI.mutate().setSpec("http: //foo.com").finalize();
   } catch (e) {
     success = e.result == Cr.NS_ERROR_MALFORMED_URI;
   }
@@ -22,10 +17,7 @@ function run_test() {
 
   success = false;
   try {
-    newURI
-      .mutate()
-      .setHost(" foo.com")
-      .finalize();
+    newURI.mutate().setHost(" foo.com").finalize();
   } catch (e) {
     success = e.result == Cr.NS_ERROR_MALFORMED_URI;
   }
