@@ -4,12 +4,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "BaseVFS.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/DebugOnly.h"
-
+#include "mozilla/SpinEventLoopUntil.h"
+#include "nsIFile.h"
+#include "nsIFileURL.h"
 #include "mozStorageService.h"
 #include "mozStorageConnection.h"
-#include "nsCollationCID.h"
 #include "nsComponentManagerUtils.h"
 #include "nsEmbedCID.h"
 #include "nsExceptionHandler.h"
@@ -17,11 +19,15 @@
 #include "mozStoragePrivateHelpers.h"
 #include "nsIObserverService.h"
 #include "nsIPropertyBag2.h"
+#include "ObfuscatingVFS.h"
+#include "QuotaVFS.h"
 #include "mozilla/Services.h"
-#include "mozilla/Preferences.h"
 #include "mozilla/LateWriteChecks.h"
 #include "mozIStorageCompletionCallback.h"
 #include "mozIStoragePendingStatement.h"
+#include "mozilla/StaticPrefs_storage.h"
+#include "mozilla/intl/Collator.h"
+#include "mozilla/intl/LocaleService.h"
 
 #include "sqlite3.h"
 #include "mozilla/AutoSQLiteLifetime.h"
@@ -31,20 +37,9 @@
 #  undef CompareString
 #endif
 
-////////////////////////////////////////////////////////////////////////////////
-//// Defines
+using mozilla::intl::Collator;
 
-#define PREF_TS_SYNCHRONOUS "toolkit.storage.synchronous"
-#define PREF_TS_SYNCHRONOUS_DEFAULT 1
-
-#define PREF_TS_PAGESIZE "toolkit.storage.pageSize"
-
-// This value must be kept in sync with the value of SQLITE_DEFAULT_PAGE_SIZE in
-// third_party/sqlite3/src/Makefile.in.
-#define PREF_TS_PAGESIZE_DEFAULT 32768
-
-namespace mozilla {
-namespace storage {
+namespace mozilla::storage {
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Memory Reporting
@@ -88,7 +83,7 @@ static void ReportConn(nsIHandleReportCallback* aHandleReport,
   path.AppendLiteral("-used");
 
   int32_t val = aConn->getSqliteRuntimeStatus(aOption);
-  aHandleReport->Callback(EmptyCString(), path, nsIMemoryReporter::KIND_HEAP,
+  aHandleReport->Callback(""_ns, path, nsIMemoryReporter::KIND_HEAP,
                           nsIMemoryReporter::UNITS_BYTES, int64_t(val), aDesc,
                           aData);
   *aTotal += val;
@@ -127,28 +122,22 @@ Service::CollectReports(nsIHandleReportCallback* aHandleReport,
 
       SQLiteMutexAutoLock lockedScope(conn->sharedDBMutex);
 
-      NS_NAMED_LITERAL_CSTRING(
-          stmtDesc,
+      constexpr auto stmtDesc =
           "Memory (approximate) used by all prepared statements used by "
-          "connections to this database.");
-      ReportConn(aHandleReport, aData, conn, pathHead,
-                 NS_LITERAL_CSTRING("stmt"), stmtDesc,
+          "connections to this database."_ns;
+      ReportConn(aHandleReport, aData, conn, pathHead, "stmt"_ns, stmtDesc,
                  SQLITE_DBSTATUS_STMT_USED, &totalConnSize);
 
-      NS_NAMED_LITERAL_CSTRING(
-          cacheDesc,
+      constexpr auto cacheDesc =
           "Memory (approximate) used by all pager caches used by connections "
-          "to this database.");
-      ReportConn(aHandleReport, aData, conn, pathHead,
-                 NS_LITERAL_CSTRING("cache"), cacheDesc,
+          "to this database."_ns;
+      ReportConn(aHandleReport, aData, conn, pathHead, "cache"_ns, cacheDesc,
                  SQLITE_DBSTATUS_CACHE_USED_SHARED, &totalConnSize);
 
-      NS_NAMED_LITERAL_CSTRING(
-          schemaDesc,
+      constexpr auto schemaDesc =
           "Memory (approximate) used to store the schema for all databases "
-          "associated with connections to this database.");
-      ReportConn(aHandleReport, aData, conn, pathHead,
-                 NS_LITERAL_CSTRING("schema"), schemaDesc,
+          "associated with connections to this database."_ns;
+      ReportConn(aHandleReport, aData, conn, pathHead, "schema"_ns, schemaDesc,
                  SQLITE_DBSTATUS_SCHEMA_USED, &totalConnSize);
     }
 
@@ -161,7 +150,7 @@ Service::CollectReports(nsIHandleReportCallback* aHandleReport,
 #endif
   }
 
-  int64_t other = ::sqlite3_memory_used() - totalConnSize;
+  int64_t other = static_cast<int64_t>(::sqlite3_memory_used() - totalConnSize);
 
   MOZ_COLLECT_REPORT("explicit/storage/sqlite/other", KIND_HEAP, UNITS_BYTES,
                      other, "All unclassified sqlite memory.");
@@ -194,29 +183,35 @@ already_AddRefed<Service> Service::getSingleton() {
   return nullptr;
 }
 
-int32_t Service::sSynchronousPref;
+int Service::AutoVFSRegistration::Init(UniquePtr<sqlite3_vfs> aVFS) {
+  MOZ_ASSERT(!mVFS);
+  if (aVFS) {
+    mVFS = std::move(aVFS);
+    return sqlite3_vfs_register(mVFS.get(), 0);
+  }
+  NS_WARNING("Failed to register VFS");
+  return SQLITE_OK;
+}
 
-// static
-int32_t Service::getSynchronousPref() { return sSynchronousPref; }
-
-int32_t Service::sDefaultPageSize = PREF_TS_PAGESIZE_DEFAULT;
+Service::AutoVFSRegistration::~AutoVFSRegistration() {
+  if (mVFS) {
+    int rc = sqlite3_vfs_unregister(mVFS.get());
+    if (rc != SQLITE_OK) {
+      NS_WARNING("Failed to unregister sqlite vfs wrapper.");
+    }
+  }
+}
 
 Service::Service()
     : mMutex("Service::mMutex"),
-      mSqliteVFS(nullptr),
       mRegistrationMutex("Service::mRegistrationMutex"),
-      mConnections() {}
+      mLastSensitivity(mozilla::intl::Collator::Sensitivity::Base) {}
 
 Service::~Service() {
   mozilla::UnregisterWeakMemoryReporter(this);
   mozilla::UnregisterStorageSQLiteDistinguishedAmount();
 
-  int rc = sqlite3_vfs_unregister(mSqliteVFS);
-  if (rc != SQLITE_OK) NS_WARNING("Failed to unregister sqlite vfs wrapper.");
-
   gService = nullptr;
-  delete mSqliteVFS;
-  mSqliteVFS = nullptr;
 }
 
 void Service::registerConnection(Connection* aConnection) {
@@ -251,12 +246,13 @@ void Service::unregisterConnection(Connection* aConnection) {
   MOZ_ASSERT(forgettingRef,
              "Attempt to unregister unknown storage connection!");
 
-  // Do not proxy the release anywhere, just let this reference drop here.  (We
+  // Do not proxy the release anywhere, just let this reference drop here. (We
   // previously did proxy the release, but that was because we invoked Close()
   // in the destructor and Close() likes to complain if it's not invoked on the
-  // opener thread, so it was essential that the last reference be dropped on
-  // the opener thread.  We now enqueue Close() inside our caller, Release(), so
-  // it doesn't actually matter what thread our reference drops on.)
+  // opener event target, so it was essential that the last reference be dropped
+  // on the opener event target.  We now enqueue Close() inside our caller,
+  // Release(), so it doesn't actually matter what thread our reference drops
+  // on.)
 }
 
 void Service::getConnections(
@@ -279,8 +275,7 @@ void Service::minimizeMemory() {
       continue;
     }
 
-    NS_NAMED_LITERAL_CSTRING(shrinkPragma, "PRAGMA shrink_memory");
-    bool onOpenedThread = false;
+    constexpr auto shrinkPragma = "PRAGMA shrink_memory"_ns;
 
     if (!conn->operationSupported(Connection::SYNCHRONOUS)) {
       // This is a mozIStorageAsyncConnection, it can only be used on the main
@@ -289,9 +284,7 @@ void Service::minimizeMemory() {
       DebugOnly<nsresult> rv = conn->ExecuteSimpleSQLAsync(
           shrinkPragma, nullptr, getter_AddRefs(ps));
       MOZ_ASSERT(NS_SUCCEEDED(rv), "Should have purged sqlite caches");
-    } else if (NS_SUCCEEDED(
-                   conn->threadOpenedOn->IsOnCurrentThread(&onOpenedThread)) &&
-               onOpenedThread) {
+    } else if (IsOnCurrentSerialEventTarget(conn->eventTargetOpenedOn)) {
       if (conn->isAsyncExecutionThreadAvailable()) {
         nsCOMPtr<mozIStoragePendingStatement> ps;
         DebugOnly<nsresult> rv = conn->ExecuteSimpleSQLAsync(
@@ -301,22 +294,21 @@ void Service::minimizeMemory() {
         conn->ExecuteSimpleSQL(shrinkPragma);
       }
     } else {
-      // We are on the wrong thread, the query should be executed on the
-      // opener thread, so we must dispatch to it.
+      // We are on the wrong event target, the query should be executed on the
+      // opener event target, so we must dispatch to it.
       // It's possible the connection is already closed or will be closed by the
-      // time our runnable runs.  ExecuteSimpleSQL will safely return with a
-      // failure in that case.  If the thread is shutting down or shut down, the
-      // dispatch will fail and that's okay.
+      // time our runnable runs. ExecuteSimpleSQL will safely return with a
+      // failure in that case. If the event target is shutting down or shut
+      // down, the dispatch will fail and that's okay.
       nsCOMPtr<nsIRunnable> event = NewRunnableMethod<const nsCString>(
           "Connection::ExecuteSimpleSQL", conn, &Connection::ExecuteSimpleSQL,
           shrinkPragma);
-      Unused << conn->threadOpenedOn->Dispatch(event, NS_DISPATCH_NORMAL);
+      Unused << conn->eventTargetOpenedOn->Dispatch(event, NS_DISPATCH_NORMAL);
     }
   }
 }
 
-sqlite3_vfs* ConstructTelemetryVFS();
-const char* GetVFSName();
+UniquePtr<sqlite3_vfs> ConstructReadOnlyNoLockVFS();
 
 static const char* sObserverTopics[] = {"memory-pressure",
                                         "xpcom-shutdown-threads"};
@@ -325,38 +317,67 @@ nsresult Service::initialize() {
   MOZ_ASSERT(NS_IsMainThread(), "Must be initialized on the main thread");
 
   int rc = AutoSQLiteLifetime::getInitResult();
-  if (rc != SQLITE_OK) return convertResultCode(rc);
+  if (rc != SQLITE_OK) {
+    return convertResultCode(rc);
+  }
 
-  mSqliteVFS = ConstructTelemetryVFS();
-  if (mSqliteVFS) {
-    rc = sqlite3_vfs_register(mSqliteVFS, 0);
-    if (rc != SQLITE_OK) return convertResultCode(rc);
-  } else {
-    NS_WARNING("Failed to register telemetry VFS");
+  /**
+   *                    The virtual file system hierarchy
+   *
+   *                                 obfsvfs
+   *                                    |
+   *                                    |
+   *                                    |
+   *                                 quotavfs
+   *                                   / \
+   *                                 /     \
+   *                               /         \
+   *                             /             \
+   *                           /                 \
+   *                     base-vfs-excl        base-vfs
+   *                          / \               / \
+   *                        /     \           /     \
+   *                      /         \       /         \
+   *                 unix-excl     win32  unix       win32
+   */
+
+  rc = mBaseSqliteVFS.Init(basevfs::ConstructVFS(false));
+  if (rc != SQLITE_OK) {
+    return convertResultCode(rc);
+  }
+
+  rc = mBaseExclSqliteVFS.Init(basevfs::ConstructVFS(true));
+  if (rc != SQLITE_OK) {
+    return convertResultCode(rc);
+  }
+
+  rc = mQuotaSqliteVFS.Init(quotavfs::ConstructVFS(basevfs::GetVFSName(
+      StaticPrefs::storage_sqlite_exclusiveLock_enabled())));
+  if (rc != SQLITE_OK) {
+    return convertResultCode(rc);
+  }
+
+  rc =
+      mObfuscatingSqliteVFS.Init(obfsvfs::ConstructVFS(quotavfs::GetVFSName()));
+  if (rc != SQLITE_OK) {
+    return convertResultCode(rc);
+  }
+
+  rc = mReadOnlyNoLockSqliteVFS.Init(ConstructReadOnlyNoLockVFS());
+  if (rc != SQLITE_OK) {
+    return convertResultCode(rc);
   }
 
   nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   NS_ENSURE_TRUE(os, NS_ERROR_FAILURE);
 
-  for (size_t i = 0; i < ArrayLength(sObserverTopics); ++i) {
-    nsresult rv = os->AddObserver(this, sObserverTopics[i], false);
+  for (auto& sObserverTopic : sObserverTopics) {
+    nsresult rv = os->AddObserver(this, sObserverTopic, false);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
   }
 
-  // We need to obtain the toolkit.storage.synchronous preferences on the main
-  // thread because the preference service can only be accessed there.  This
-  // is cached in the service for all future Open[Unshared]Database calls.
-  sSynchronousPref =
-      Preferences::GetInt(PREF_TS_SYNCHRONOUS, PREF_TS_SYNCHRONOUS_DEFAULT);
-
-  // We need to obtain the toolkit.storage.pageSize preferences on the main
-  // thread because the preference service can only be accessed there.  This
-  // is cached in the service for all future Open[Unshared]Database calls.
-  sDefaultPageSize =
-      Preferences::GetInt(PREF_TS_PAGESIZE, PREF_TS_PAGESIZE_DEFAULT);
-
   mozilla::RegisterWeakMemoryReporter(this);
   mozilla::RegisterStorageSQLiteDistinguishedAmount(
       StorageSQLiteDistinguishedAmount);
@@ -366,69 +387,86 @@ nsresult Service::initialize() {
 
 int Service::localeCompareStrings(const nsAString& aStr1,
                                   const nsAString& aStr2,
-                                  int32_t aComparisonStrength) {
-  // The implementation of nsICollation.CompareString() is platform-dependent.
-  // On Linux it's not thread-safe.  It may not be on Windows and OS X either,
-  // but it's more difficult to tell.  We therefore synchronize this method.
+                                  Collator::Sensitivity aSensitivity) {
+  // The mozilla::intl::Collator is not thread safe, since the Collator::Options
+  // can be changed.
   MutexAutoLock mutex(mMutex);
 
-  nsICollation* coll = getLocaleCollation();
-  if (!coll) {
+  Collator* collator = getCollator();
+  if (!collator) {
     NS_ERROR("Storage service has no collation");
     return 0;
   }
 
-  int32_t res;
-  nsresult rv = coll->CompareString(aComparisonStrength, aStr1, aStr2, &res);
-  if (NS_FAILED(rv)) {
-    NS_ERROR("Collation compare string failed");
-    return 0;
+  if (aSensitivity != mLastSensitivity) {
+    Collator::Options options{};
+    options.sensitivity = aSensitivity;
+    auto result = mCollator->SetOptions(options);
+
+    if (result.isErr()) {
+      NS_WARNING("Could not configure the mozilla::intl::Collation.");
+      return 0;
+    }
+    mLastSensitivity = aSensitivity;
   }
 
-  return res;
+  return collator->CompareStrings(aStr1, aStr2);
 }
 
-nsICollation* Service::getLocaleCollation() {
+Collator* Service::getCollator() {
   mMutex.AssertCurrentThreadOwns();
 
-  if (mLocaleCollation) return mLocaleCollation;
+  if (mCollator) {
+    return mCollator.get();
+  }
 
-  nsCOMPtr<nsICollationFactory> collFact =
-      do_CreateInstance(NS_COLLATIONFACTORY_CONTRACTID);
-  if (!collFact) {
-    NS_WARNING("Could not create collation factory");
+  auto result = mozilla::intl::LocaleService::TryCreateComponent<Collator>();
+  if (result.isErr()) {
+    NS_WARNING("Could not create mozilla::intl::Collation.");
     return nullptr;
   }
 
-  nsresult rv = collFact->CreateCollation(getter_AddRefs(mLocaleCollation));
-  if (NS_FAILED(rv)) {
-    NS_WARNING("Could not create collation");
+  mCollator = result.unwrap();
+
+  // Sort in a case-insensitive way, where "base" letters are considered
+  // equal, e.g: a = á, a = A, a ≠ b.
+  Collator::Options options{};
+  options.sensitivity = Collator::Sensitivity::Base;
+  auto optResult = mCollator->SetOptions(options);
+
+  if (optResult.isErr()) {
+    NS_WARNING("Could not configure the mozilla::intl::Collation.");
+    mCollator = nullptr;
     return nullptr;
   }
 
-  return mLocaleCollation;
+  return mCollator.get();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 //// mozIStorageService
 
 NS_IMETHODIMP
-Service::OpenSpecialDatabase(const char* aStorageKey,
+Service::OpenSpecialDatabase(const nsACString& aStorageKey,
+                             const nsACString& aName, uint32_t aConnectionFlags,
                              mozIStorageConnection** _connection) {
-  nsresult rv;
-
-  nsCOMPtr<nsIFile> storageFile;
-  if (::strcmp(aStorageKey, "memory") == 0) {
-    // just fall through with nullptr storageFile, this will cause the storage
-    // connection to use a memory DB.
-  } else {
+  if (!aStorageKey.Equals(kMozStorageMemoryStorageKey)) {
     return NS_ERROR_INVALID_ARG;
   }
 
-  RefPtr<Connection> msc =
-      new Connection(this, SQLITE_OPEN_READWRITE, Connection::SYNCHRONOUS);
+  const bool interruptible =
+      aConnectionFlags & mozIStorageService::CONNECTION_INTERRUPTIBLE;
 
-  rv = storageFile ? msc->initialize(storageFile) : msc->initialize();
+  int flags = SQLITE_OPEN_READWRITE;
+
+  if (!aName.IsEmpty()) {
+    flags |= SQLITE_OPEN_URI;
+  }
+
+  RefPtr<Connection> msc =
+      new Connection(this, flags, Connection::SYNCHRONOUS,
+                     kMozStorageMemoryStorageKey, interruptible);
+  const nsresult rv = msc->initialize(aStorageKey, aName);
   NS_ENSURE_SUCCESS(rv, rv);
 
   msc.forget(_connection);
@@ -459,7 +497,7 @@ class AsyncInitDatabase final : public Runnable {
 
     if (mGrowthIncrement >= 0) {
       // Ignore errors. In the future, we might wish to log them.
-      (void)mConnection->SetGrowthIncrement(mGrowthIncrement, EmptyCString());
+      (void)mConnection->SetGrowthIncrement(mGrowthIncrement, ""_ns);
     }
 
     return DispatchResult(
@@ -494,8 +532,8 @@ class AsyncInitDatabase final : public Runnable {
 }  // namespace
 
 NS_IMETHODIMP
-Service::OpenAsyncDatabase(nsIVariant* aDatabaseStore,
-                           nsIPropertyBag2* aOptions,
+Service::OpenAsyncDatabase(nsIVariant* aDatabaseStore, uint32_t aOpenFlags,
+                           uint32_t /* aConnectionFlags */,
                            mozIStorageCompletionCallback* aCallback) {
   if (!NS_IsMainThread()) {
     return NS_ERROR_NOT_SAME_THREAD;
@@ -503,43 +541,17 @@ Service::OpenAsyncDatabase(nsIVariant* aDatabaseStore,
   NS_ENSURE_ARG(aDatabaseStore);
   NS_ENSURE_ARG(aCallback);
 
-  nsresult rv;
-  bool shared = false;
-  bool readOnly = false;
-  bool ignoreLockingMode = false;
-  int32_t growthIncrement = -1;
-
-#define FAIL_IF_SET_BUT_INVALID(rv)                    \
-  if (NS_FAILED(rv) && rv != NS_ERROR_NOT_AVAILABLE) { \
-    return NS_ERROR_INVALID_ARG;                       \
-  }
-
-  // Deal with options first:
-  if (aOptions) {
-    rv = aOptions->GetPropertyAsBool(NS_LITERAL_STRING("readOnly"), &readOnly);
-    FAIL_IF_SET_BUT_INVALID(rv);
-
-    rv = aOptions->GetPropertyAsBool(NS_LITERAL_STRING("ignoreLockingMode"),
-                                     &ignoreLockingMode);
-    FAIL_IF_SET_BUT_INVALID(rv);
-    // Specifying ignoreLockingMode will force use of the readOnly flag:
-    if (ignoreLockingMode) {
-      readOnly = true;
-    }
-
-    rv = aOptions->GetPropertyAsBool(NS_LITERAL_STRING("shared"), &shared);
-    FAIL_IF_SET_BUT_INVALID(rv);
-
-    // NB: we re-set to -1 if we don't have a storage file later on.
-    rv = aOptions->GetPropertyAsInt32(NS_LITERAL_STRING("growthIncrement"),
-                                      &growthIncrement);
-    FAIL_IF_SET_BUT_INVALID(rv);
-  }
+  const bool shared = aOpenFlags & mozIStorageService::OPEN_SHARED;
+  const bool ignoreLockingMode =
+      aOpenFlags & mozIStorageService::OPEN_IGNORE_LOCKING_MODE;
+  // Specifying ignoreLockingMode will force use of the readOnly flag:
+  const bool readOnly =
+      ignoreLockingMode || (aOpenFlags & mozIStorageService::OPEN_READONLY);
   int flags = readOnly ? SQLITE_OPEN_READONLY : SQLITE_OPEN_READWRITE;
 
   nsCOMPtr<nsIFile> storageFile;
   nsCOMPtr<nsISupports> dbStore;
-  rv = aDatabaseStore->GetAsISupports(getter_AddRefs(dbStore));
+  nsresult rv = aDatabaseStore->GetAsISupports(getter_AddRefs(dbStore));
   if (NS_SUCCEEDED(rv)) {
     // Generally, aDatabaseStore holds the database nsIFile.
     storageFile = do_QueryInterface(dbStore, &rv);
@@ -563,7 +575,7 @@ Service::OpenAsyncDatabase(nsIVariant* aDatabaseStore,
     // Sometimes, however, it's a special database name.
     nsAutoCString keyString;
     rv = aDatabaseStore->GetAsACString(keyString);
-    if (NS_FAILED(rv) || !keyString.EqualsLiteral("memory")) {
+    if (NS_FAILED(rv) || !keyString.Equals(kMozStorageMemoryStorageKey)) {
       return NS_ERROR_INVALID_ARG;
     }
 
@@ -571,34 +583,44 @@ Service::OpenAsyncDatabase(nsIVariant* aDatabaseStore,
     // connection to use a memory DB.
   }
 
-  if (!storageFile && growthIncrement >= 0) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
   // Create connection on this thread, but initialize it on its helper thread.
+  nsAutoCString telemetryFilename;
+  if (!storageFile) {
+    telemetryFilename.Assign(kMozStorageMemoryStorageKey);
+  } else {
+    rv = storageFile->GetNativeLeafName(telemetryFilename);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
   RefPtr<Connection> msc =
-      new Connection(this, flags, Connection::ASYNCHRONOUS, ignoreLockingMode);
+      new Connection(this, flags, Connection::ASYNCHRONOUS, telemetryFilename,
+                     /* interruptible */ true, ignoreLockingMode);
   nsCOMPtr<nsIEventTarget> target = msc->getAsyncExecutionTarget();
   MOZ_ASSERT(target,
              "Cannot initialize a connection that has been closed already");
 
-  RefPtr<AsyncInitDatabase> asyncInit =
-      new AsyncInitDatabase(msc, storageFile, growthIncrement, aCallback);
+  RefPtr<AsyncInitDatabase> asyncInit = new AsyncInitDatabase(
+      msc, storageFile, /* growthIncrement */ -1, aCallback);
   return target->Dispatch(asyncInit, nsIEventTarget::DISPATCH_NORMAL);
 }
 
 NS_IMETHODIMP
-Service::OpenDatabase(nsIFile* aDatabaseFile,
+Service::OpenDatabase(nsIFile* aDatabaseFile, uint32_t aConnectionFlags,
                       mozIStorageConnection** _connection) {
   NS_ENSURE_ARG(aDatabaseFile);
 
+  const bool interruptible =
+      aConnectionFlags & mozIStorageService::CONNECTION_INTERRUPTIBLE;
+
   // Always ensure that SQLITE_OPEN_CREATE is passed in for compatibility
   // reasons.
-  int flags =
+  const int flags =
       SQLITE_OPEN_READWRITE | SQLITE_OPEN_SHAREDCACHE | SQLITE_OPEN_CREATE;
-  RefPtr<Connection> msc = new Connection(this, flags, Connection::SYNCHRONOUS);
-
-  nsresult rv = msc->initialize(aDatabaseFile);
+  nsAutoCString telemetryFilename;
+  nsresult rv = aDatabaseFile->GetNativeLeafName(telemetryFilename);
+  NS_ENSURE_SUCCESS(rv, rv);
+  RefPtr<Connection> msc = new Connection(this, flags, Connection::SYNCHRONOUS,
+                                          telemetryFilename, interruptible);
+  rv = msc->initialize(aDatabaseFile);
   NS_ENSURE_SUCCESS(rv, rv);
 
   msc.forget(_connection);
@@ -606,17 +628,23 @@ Service::OpenDatabase(nsIFile* aDatabaseFile,
 }
 
 NS_IMETHODIMP
-Service::OpenUnsharedDatabase(nsIFile* aDatabaseFile,
+Service::OpenUnsharedDatabase(nsIFile* aDatabaseFile, uint32_t aConnectionFlags,
                               mozIStorageConnection** _connection) {
   NS_ENSURE_ARG(aDatabaseFile);
 
+  const bool interruptible =
+      aConnectionFlags & mozIStorageService::CONNECTION_INTERRUPTIBLE;
+
   // Always ensure that SQLITE_OPEN_CREATE is passed in for compatibility
   // reasons.
-  int flags =
+  const int flags =
       SQLITE_OPEN_READWRITE | SQLITE_OPEN_PRIVATECACHE | SQLITE_OPEN_CREATE;
-  RefPtr<Connection> msc = new Connection(this, flags, Connection::SYNCHRONOUS);
-
-  nsresult rv = msc->initialize(aDatabaseFile);
+  nsAutoCString telemetryFilename;
+  nsresult rv = aDatabaseFile->GetNativeLeafName(telemetryFilename);
+  NS_ENSURE_SUCCESS(rv, rv);
+  RefPtr<Connection> msc = new Connection(this, flags, Connection::SYNCHRONOUS,
+                                          telemetryFilename, interruptible);
+  rv = msc->initialize(aDatabaseFile);
   NS_ENSURE_SUCCESS(rv, rv);
 
   msc.forget(_connection);
@@ -625,56 +653,38 @@ Service::OpenUnsharedDatabase(nsIFile* aDatabaseFile,
 
 NS_IMETHODIMP
 Service::OpenDatabaseWithFileURL(nsIFileURL* aFileURL,
+                                 const nsACString& aTelemetryFilename,
+                                 uint32_t aConnectionFlags,
                                  mozIStorageConnection** _connection) {
   NS_ENSURE_ARG(aFileURL);
 
+  const bool interruptible =
+      aConnectionFlags & mozIStorageService::CONNECTION_INTERRUPTIBLE;
+
   // Always ensure that SQLITE_OPEN_CREATE is passed in for compatibility
   // reasons.
-  int flags = SQLITE_OPEN_READWRITE | SQLITE_OPEN_SHAREDCACHE |
-              SQLITE_OPEN_CREATE | SQLITE_OPEN_URI;
-  RefPtr<Connection> msc = new Connection(this, flags, Connection::SYNCHRONOUS);
-
-  nsresult rv = msc->initialize(aFileURL);
+  const int flags = SQLITE_OPEN_READWRITE | SQLITE_OPEN_SHAREDCACHE |
+                    SQLITE_OPEN_CREATE | SQLITE_OPEN_URI;
+  nsresult rv;
+  nsAutoCString telemetryFilename;
+  if (!aTelemetryFilename.IsEmpty()) {
+    telemetryFilename = aTelemetryFilename;
+  } else {
+    nsCOMPtr<nsIFile> databaseFile;
+    rv = aFileURL->GetFile(getter_AddRefs(databaseFile));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = databaseFile->GetNativeLeafName(telemetryFilename);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  RefPtr<Connection> msc = new Connection(this, flags, Connection::SYNCHRONOUS,
+                                          telemetryFilename, interruptible);
+  rv = msc->initialize(aFileURL);
   NS_ENSURE_SUCCESS(rv, rv);
 
   msc.forget(_connection);
   return NS_OK;
 }
 
-NS_IMETHODIMP
-Service::BackupDatabaseFile(nsIFile* aDBFile, const nsAString& aBackupFileName,
-                            nsIFile* aBackupParentDirectory, nsIFile** backup) {
-  nsresult rv;
-  nsCOMPtr<nsIFile> parentDir = aBackupParentDirectory;
-  if (!parentDir) {
-    // This argument is optional, and defaults to the same parent directory
-    // as the current file.
-    rv = aDBFile->GetParent(getter_AddRefs(parentDir));
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  nsCOMPtr<nsIFile> backupDB;
-  rv = parentDir->Clone(getter_AddRefs(backupDB));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = backupDB->Append(aBackupFileName);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = backupDB->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsAutoString fileName;
-  rv = backupDB->GetLeafName(fileName);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = backupDB->Remove(false);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  backupDB.forget(backup);
-
-  return aDBFile->CopyTo(parentDir, fileName);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 //// nsIObserver
 
@@ -692,21 +702,23 @@ Service::Observe(nsISupports*, const char* aTopic, const char16_t*) {
 
     nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
 
-    for (size_t i = 0; i < ArrayLength(sObserverTopics); ++i) {
-      (void)os->RemoveObserver(this, sObserverTopics[i]);
+    for (auto& sObserverTopic : sObserverTopics) {
+      (void)os->RemoveObserver(this, sObserverTopic);
     }
 
-    SpinEventLoopUntil([&]() -> bool {
-      // We must wait until all the closing connections are closed.
-      nsTArray<RefPtr<Connection>> connections;
-      getConnections(connections);
-      for (auto& conn : connections) {
-        if (conn->isClosing()) {
-          return false;
-        }
-      }
-      return true;
-    });
+    SpinEventLoopUntil("storage::Service::Observe(xpcom-shutdown-threads)"_ns,
+                       [&]() -> bool {
+                         // We must wait until all the closing connections are
+                         // closed.
+                         nsTArray<RefPtr<Connection>> connections;
+                         getConnections(connections);
+                         for (auto& conn : connections) {
+                           if (conn->isClosing()) {
+                             return false;
+                           }
+                         }
+                         return true;
+                       });
 
 #ifdef DEBUG
     nsTArray<RefPtr<Connection>> connections;
@@ -715,7 +727,7 @@ Service::Observe(nsISupports*, const char* aTopic, const char16_t*) {
       if (!connections[i]->isClosed()) {
         // getFilename is only the leaf name for the database file,
         // so it shouldn't contain privacy-sensitive information.
-        CrashReporter::AnnotateCrashReport(
+        CrashReporter::RecordAnnotationNSCString(
             CrashReporter::Annotation::StorageConnectionNotClosed,
             connections[i]->getFilename());
         printf_stderr("Storage connection not closed: %s",
@@ -729,5 +741,4 @@ Service::Observe(nsISupports*, const char* aTopic, const char16_t*) {
   return NS_OK;
 }
 
-}  // namespace storage
-}  // namespace mozilla
+}  // namespace mozilla::storage
