@@ -12,17 +12,12 @@ cd $GECKO_PATH
 export PATH=$PATH:$MOZ_FETCHES_DIR/rustc/bin:$HOME/.cargo/bin
 cd gfx/wr/
 mkdir .cargo
-cargo vendor --sync ./Cargo.toml > .cargo/config
+cargo vendor --locked --sync ./Cargo.toml > .cargo/config.toml
 mkdir wrench-deps
 mv vendor .cargo wrench-deps/
-mkdir wrench-deps/cargo-apk
-# Until there's a version of cargo-apk published on crates.io that has
-# https://github.com/rust-windowing/android-rs-glue/pull/223, we need to use
-# an unpublished version.
-cargo install --path $MOZ_FETCHES_DIR/android-rs-glue/cargo-apk --root wrench-deps/cargo-apk cargo-apk
 
 ci-scripts/install-meson.sh
 mv meson wrench-deps/meson
 
 mkdir -p $UPLOAD_DIR
-tar caf $UPLOAD_DIR/wrench-deps.tar.bz2 wrench-deps
+tar caf $UPLOAD_DIR/wrench-deps.tar.zst wrench-deps
